import unittest
import os
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Poscar
from jarvis.vasp.joptb88vdw import *  # Auto_Kpoints
from jarvis.lammps.jlammps import read_data
from jarvis.sklearn.get_desc import get_comp_descp
from jarvis.tools.vasp import *
from jarvis.slme.slme import *
from jarvis.boltztrap.boltztrap import *
from jarvis.elastic_tens.vasp import *
from jarvis.ip_optics.freq_dielectric import *

pos = os.path.join(
    os.path.dirname(__file__), "..", "vasp", "examples", "SiOptb88", "POSCAR"
)
dir = os.path.join(os.path.dirname(__file__), "..", "vasp", "examples", "SiOptb88")
run = os.path.join(
    os.path.dirname(__file__),
    "..",
    "vasp",
    "examples",
    "SiOptb88",
    "MAIN-BAND-bulk@mp_149",
    "vasprun.xml",
)
kpfile = os.path.join(
    os.path.dirname(__file__),
    "..",
    "vasp",
    "examples",
    "SiOptb88",
    "MAIN-BAND-bulk@mp_149",
    "KPOINTS",
)
out = os.path.join(
    os.path.dirname(__file__),
    "..",
    "vasp",
    "examples",
    "SiOptb88",
    "MAIN-ELASTIC-bulk@mp_149",
    "OUTCAR",
)
mbjrun = os.path.join(
    os.path.dirname(__file__),
    "..",
    "vasp",
    "examples",
    "SiOptb88",
    "MAIN-MBJ-bulk@mp_149",
    "vasprun.xml",
)
mainrun = os.path.join(
    os.path.dirname(__file__),
    "..",
    "vasp",
    "examples",
    "SiOptb88",
    "MAIN-RELAX-bulk@mp_149",
    "vasprun.xml",
)


def sample_structure():
    poscar = Poscar.from_file(pos)
    return poscar


def test_read_poscar():
    poscar = Structure.from_file(pos)
    assert len(poscar) == 2


def test_Auto_kpoints():
    poscar = Poscar.from_file(pos)
    kp = list(Auto_Kpoints(mat=poscar, length=20).kpts[0])
    assert kp == [6, 6, 6]


def test_check_polar():
    p = sample_structure().structure
    surfaces = surfer(mat=p, write_file=False)
    s = (surfaces[0]).structure
    print(check_polar(s))
    assert check_polar(s) == False


# def test_get_lowest_en_from_mp():
#    en_atom=get_lowest_en_from_mp('Al')
#    assert round(float(en_atom),4) == round(float(-3.74810318),4)


def test_make_big():
    p = sample_structure()
    strt = p.structure.copy()
    big_p = make_big(poscar=p)
    assert len(strt) * 3 ** 3 == len(big_p.structure)


def test_converg_encut():
    p = sample_structure()
    cwd = str(os.getcwd())
    os.chdir(dir)
    final_enc = converg_encut(mat=p)
    os.chdir(cwd)
    assert final_enc == 500


def test_converg_kpoints():
    p = sample_structure()
    cwd = str(os.getcwd())
    os.chdir(dir)
    final_kp = converg_kpoints(mat=p)
    os.chdir(cwd)
    assert final_kp == 35


def test_smart_converge():
    p = sample_structure()
    cwd = str(os.getcwd())
    os.chdir(dir)
    en, mat_f = smart_converge(mat=p)
    assert float(en) == float(-8.3381172)


def test_bandgap1():
    eg, dir = bandgap(vrun=run)
    assert eg == 0.006000000000000227


def test_bandgap2():
    vrun = str(run).replace("BAND", "RELAX")
    eg, dir = bandgap(vrun=vrun)
    # print (eg,dir)
    assert eg == 0.14800000000000058


def test_bandstr():
    bplot = bandstr(vrun=run, kpfile=kpfile)
    # bplot.savefig('pp.png')
    assert (bplot.xlim()[0]) == 0
    bplot.close()


def test_plot_dos():
    plt1, plt2, plt3 = plot_dos(vrun=run)
    assert (plt1.xlim()[0]) == -5
    plt1.close()
    plt2.close()
    plt3.close()


# def test_plot_kp_convergence():
#    _,kp=plot_kp_convergence(dir)
#    assert kp=='11x11x11'


def test_plot_enc_convergence():
    _, enc = plot_enc_convergence(dir)
    assert enc == 500


def test_ip_optics():
    vrun = str(run).replace("BAND", "OPTICS")
    op = ip_optics(vrun)
    val = op["real_part"][0][0]
    assert val == 13.4865


def test_elastic_props():
    info = elastic_props(outcar=out)
    assert (info["KV"]) == 87.267


def test_magnetic_moment():
    osz = str(run).replace("BAND", "RELAX").replace("vasprun.xml", "OSZICAR")
    m = magnetic_moment(osz)
    assert m == 0


def test_get_spacegroup():
    strt = Structure.from_file(pos)
    num, symb = get_spacegroup(strt)
    assert symb == "Fd-3m"


def test_get_area():
    strt = Structure.from_file(pos)
    ar = get_area(strt)
    assert ar == 12.950104933895442


def test_slme():
    en, abz, dirgap, indirgap = optics(mbjrun)
    abz = abz * 100.0
    SLME = slme(en, abz, indirgap, indirgap, plot_current_voltage=False)
    SQ = calculate_SQ(indirgap)
    assert SLME == 0.3323125002699776


# def test_boltztrap():
#   b=boltz_run(mainrun)
#   val=get_prop(b,prop='zt')
#   assert val[0]==1.5913528701329165



"""
Code to calculate spin-orbit spillage.
Please cite the following paper if you use the code:
Kamal Choudhary, Kevin F. Garrity, Francesca Tavazza, Scientific Reports, 9,8534 (2019)
https://www.nature.com/articles/s41598-019-45028-y
"""

import sys
import numpy as np
from jarvis.spillage.vaspwfc import vaspwfc
import scipy as sp


def isclose(n1, n2, rel_tol=1e-7):

    if abs(n1 - n2) < rel_tol:
        return True
    else:
        return False


def orth(A):
    u, s, vh = np.linalg.svd(A, full_matrices=False)
    M, N = A.shape
    eps = np.finfo(float).eps
    tol = max(M, N) * np.amax(s) * eps
    num = np.sum(s > tol, dtype=int)
    Q = u[:, :num]
    return Q, num


def overlap_so_spinpol(wf_noso, wf_so):
    noso = vaspwfc(fnm=wf_noso, lsorbit=False)
    noso.printWF()
    so = vaspwfc(fnm=wf_so, lsorbit=True)
    so.printWF()
    # band gap stuff. not needed per se, just a useful sanity check
    noso_k, noso_bands = noso.readWFBand()
    so_k, so_bands = so.readWFBand()

    # changes section 1
    nelec_list = []
    for nk1 in range(1, noso._nkpts + 1):  # no spin orbit kpoints loop
        knoso = noso._kvecs[nk1 - 1, :]
        for nk2 in range(1, so._nkpts + 1):  # spin orbit
            kso = so._kvecs[nk2 - 1, :]
            if (
                isclose(kso[0], knoso[0])
                and isclose(kso[1], knoso[1])
                and isclose(kso[2], knoso[2])
            ):  # do kpoints match?
                for c, e in enumerate(noso._occs[0, nk1 - 1, :]):
                    if e < 0.5:
                        cup = c
                        break
                for c, e in enumerate(noso._occs[1, nk1 - 1, :]):
                    if e < 0.5:
                        cdn = c
                        break

                nelec_list.append([cup, cdn, cup + cdn])

    n_arr = np.array(nelec_list)

    n_up = int(round(np.mean(n_arr[:, 0])))
    n_dn = int(round(np.mean(n_arr[:, 1])))
    n_tot = int(round(np.mean(n_arr[:, 2])))

    nelec = int(n_tot)

    noso_homo_up = np.max(noso_bands[0, :, n_up - 1])
    noso_lumo_up = np.min(noso_bands[0, :, n_up])

    noso_homo_dn = np.max(noso_bands[1, :, n_dn - 1])
    noso_lumo_dn = np.min(noso_bands[1, :, n_dn])

    so_homo = np.max(so_bands[0, :, nelec - 1])
    so_lumo = np.min(so_bands[0, :, nelec])

    noso_direct_up = np.min(noso_bands[0, :, n_up] - noso_bands[0, :, n_up - 1])
    noso_direct_dn = np.min(noso_bands[1, :, n_dn] - noso_bands[1, :, n_dn - 1])

    so_direct = np.min(so_bands[0, :, nelec] - so_bands[0, :, nelec - 1])

    noso_direct = 1000000.0
    noso_homo = -10000000.0
    noso_lumo = 100000000.0
    for i in range(noso_bands.shape[1]):
        homo_k = max(noso_bands[0, i, n_up - 1], noso_bands[1, i, n_dn - 1])
        lumo_k = min(noso_bands[0, i, n_up], noso_bands[1, i, n_dn])
        noso_direct = min(noso_direct, lumo_k - homo_k)

        noso_homo = max(noso_homo, homo_k)
        noso_lumo = min(noso_lumo, lumo_k)

    gamma_k = []
    kpoints = []
    np.set_printoptions(precision=4)

    x = []
    y = []

    nelec_tot = 0.0
    for nk1 in range(1, noso._nkpts + 1):  # no spin orbit kpoints loop
        knoso = noso._kvecs[nk1 - 1, :]
        for nk2 in range(1, so._nkpts + 1):  # spin orbit
            kso = so._kvecs[nk2 - 1, :]
            if (
                isclose(kso[0], knoso[0])
                and isclose(kso[1], knoso[1])
                and isclose(kso[2], knoso[2])
            ):  # do kpoints match?

                # changes section 2
                nelec_up = n_arr[nk1 - 1, 0]
                nelec_dn = n_arr[nk1 - 1, 1]
                nelec_tot = n_arr[nk1 - 1, 2]

                gamma_k.append(nelec_tot)
                kpoints.append(kso)
                Mmn = 0.0
                vnoso = noso.readBandCoeff(ispin=1, ikpt=nk1, iband=1, norm=False)
                n_noso1 = vnoso.shape[0]
                vnoso = noso.readBandCoeff(ispin=2, ikpt=nk1, iband=1, norm=False)
                n_noso2 = vnoso.shape[0]
                vso = so.readBandCoeff(ispin=1, ikpt=nk2, iband=1, norm=False)
                n_so = vso.shape[0]
                vs = min(n_noso1 * 2, n_so)
                Vnoso = np.zeros((vs, nelec_tot), dtype=complex)
                Vso = np.zeros((vs, nelec_tot), dtype=complex)

                # prepare matricies
                for n1 in range(1, nelec_up + 1):
                    Vnoso[0 : vs // 2, n1 - 1] = noso.readBandCoeff(
                        ispin=1, ikpt=nk1, iband=n1, norm=False
                    )[0 : vs // 2]
                for n1 in range(1, nelec_dn + 1):
                    Vnoso[vs // 2 : vs, n1 - 1 + nelec_up] = noso.readBandCoeff(
                        ispin=2, ikpt=nk1, iband=n1, norm=False
                    )[0 : vs // 2]

                for n1 in range(1, nelec_tot + 1):
                    t = so.readBandCoeff(ispin=1, ikpt=nk2, iband=n1, norm=False)
                    Vso[0 : vs // 2, n1 - 1] = t[0 : vs // 2]
                    Vso[vs // 2 : vs, n1 - 1] = t[n_so // 2 : n_so // 2 + vs // 2]
                Qnoso, num_noso = orth(Vnoso)  # make orthonormal basis?

                Qso, num_so = orth(Vso)

                # evalute main expression

                a = []
                for n1 in range(0, nelec_tot):  # noso occupied bands
                    v1 = Qnoso[:, n1]
                    aa = 0.0
                    for n2 in range(0, nelec_tot):  # so occupied bands
                        v2 = Qso[:, n2]

                        t = np.dot(np.conj(v1), v2)
                        Mmn += t * t.conj()
                        aa += (t * t.conj()).real
                    a.append(aa)
                gamma_k[-1] -= Mmn  # eq 4 in prb 90 125133
                x.append(kso)
                y.append(np.real(gamma_k[-1]))
                if gamma_k[-1] > 0.5:
                    print(
                        "nk1 nk2 kpoint gamma_k ",
                        nk1,
                        nk2,
                        kso,
                        knoso,
                        np.real(gamma_k[-1]),
                        "!!!!!!!!!!",
                    )

    gmax = max(np.real(gamma_k))
    nkmax = np.argmax(np.real(gamma_k))
    kmax = kpoints[nkmax]

    print("------------------------------------")
    print()
    print("                   INDIRECT DIRECT      HOMO/LUMO (eV)")
    print(
        "no spin-orbit gaps",
        "{:+.3f}".format(float(noso_lumo - noso_homo)),
        "{:+.3f}".format(noso_direct),
        "   ",
        [noso_homo, noso_lumo],
    )
    print(
        "spin-orbit gaps   ",
        "{:+.3f}".format(float(so_lumo - so_homo)),
        "{:+.3f}".format(so_direct),
        "   ",
        [so_homo, so_lumo],
    )
    print()

    print("gamma max", np.real(gmax), " at k =  ", kmax)

    return (
        np.real(gmax),
        np.real(gamma_k),
        kmax,
        kpoints,
        noso_direct,
        so_direct,
        x,
        y,
        so_lumo,
        so_homo,
        noso_lumo,
        noso_homo,
    )


def gaps_only(nelec, wf):
    noso = vaspwfc(fnm=wf)
    noso.printWF()
    noso_k, noso_bands = noso.readWFBand()
    so_k, so_bands = so.readWFBand()
    noso_homo = np.max(noso_bands[0, :, nelec - 1])
    noso_lumo = np.min(noso_bands[0, :, nelec])


if __name__ == "__main__":
    # JVASP-1044

    wf_noso = "WAVECAR.noso"  # sys.argv[1]
    wf_so = "WAVECAR.so"  # sys.argv[2]
    gamma_max, gamma_k, kmax, kpoints, noso_direct, so_direct, x, y, so_lumo, so_homo, noso_lumo, noso_homo = overlap_so_spinpol(
        wf_noso, wf_so
    )
    print(gamma_max)



import glob
import os

from setuptools import setup, find_packages

JARVIS_DIR = os.path.dirname(os.path.abspath(__file__))

base_dir = os.path.dirname(__file__)
with open(os.path.join(base_dir, "README.rst")) as f:
    long_d = f.read()

setup(
    name="jarvis-tools",
    version="2019.07.22.1",

    long_description=long_d,
    long_description_content_type='text/markdown',
    install_requires=[
        "numpy==1.16.3",
        "scipy==1.2.1",
        "pymatgen>=2018.12.12",
        "custodian",
        "ase==3.11.0",
        "scikit-learn",
        "interruptingcow>=0.8",
        "pybtex>=0.21",
        "blob",
        "toolz",
        "fortranformat>=0.2.5",
      
    ],
    package_data={"jarvis.sklearn":['element_charge.json','Elements.json']},
    extras_require={
        "babel": ["openbabel", "pybel"],
        "doc": ["sphinx>=1.3.1", "sphinx-rtd-theme>=0.1.8"],
    },
    author="Kamal Choudhary",
    author_email="kamal.choudhary@nist.gov",
    description=(
        "High throughput computation with density functional theory, molecular dynamics and machine learning. https://jarvis.nist.gov/"
    ),
    license="MIT",
    url="https://github.com/usnistgov/jarvis",
    packages=find_packages(),
    #long_description=open(os.path.join(os.path.dirname(__file__), "README.rst")).read(),
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
    # scripts=glob.glob(os.path.join(JARVIS_DIR,  "*"))
)



import unittest
import os
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Poscar
from jarvis.lammps.jlammps import read_data
from jarvis.plane_defect.surface import surfer, pmg_surfer
from jarvis.point_defect.vacancy import vac_antisite_def_struct_gen

dat = os.path.join(
    os.path.dirname(__file__),
    "..",
    "lammps",
    "examples",
    "Al03.eam.alloy_nist",
    "bulk@mp-134_fold",
    "mp-134",
    "data",
)
ff = os.path.join(
    os.path.dirname(__file__),
    "..",
    "lammps",
    "examples",
    "Al03.eam.alloy_nist",
    "bulk@mp-134_fold",
    "mp-134",
    "potential.mod",
)


def sample_strt():
    data = read_data(data=dat, ff=ff)
    return data


def test_vac_antisite_def_struct_gen():
    data = read_data(data=dat, ff=ff)
    vacs = vac_antisite_def_struct_gen(struct=data, c_size=0, write_file=False)
    assert len(vacs) == 2


def test_pmg_surfer():
    s = sample_strt()
    surf = len(pmg_surfer(mat=s, write_file=False))
    assert surf == 4



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import numpy as np
from math import sqrt
from jarvis.spillage.vasp_constant import *
from scipy.fftpack import fftfreq, fftn, ifftn

############################################################
"""
This program is based on the code written by Ren Hao <renh@upc.edu.cn>.
"""


class vaspwfc:
    """
    Class for VASP Pseudowavefunction stored in WAVECAR

    The format of VASP WAVECAR, as shown in
        http://www.andrew.cmu.edu/user/feenstra/wavetrans/
    is:
        Record-length #spin components RTAG(a value specifying the precision)
        #k-points #bands ENCUT(maximum energy for plane waves)
        LatVec-A
        LatVec-B
        LatVec-C
        Loop over spin
           Loop over k-points
              #plane waves, k vector
              Loop over bands
                 band energy, band occupation
              End loop over bands
              Loop over bands
                 Loop over plane waves
                    Plane-wave coefficient
                 End loop over plane waves
              End loop over bands
           End loop over k-points
        End loop over spin
    """

    def __init__(self, fnm="WAVECAR", lsorbit=False, lgamma=False):
        """
        Initialization.
        """

        self._fname = fnm
        self._lsoc = lsorbit
        self._lgam = lgamma

        assert not (lsorbit and lgamma), "The two settings conflict!"

        try:
            self._wfc = open(self._fname, "rb")
        except:
            raise IOError("Failed to open %s" % self._fname)

        # read the basic information
        self.readWFHeader()
        # read the band information
        self.readWFBand()

        if self._lsoc:
            assert self._nspin == 1, "NSPIN = 1 for noncollinear version WAVECAR!"

    def isSocWfc(self):
        """
        Is the WAVECAR from an SOC calculation?
        """
        return True if self._lsoc else False

    def printWF(self):
        """
        print 'KFG modification'
        print
        print 
        print 'nkpts', self._nkpts 
        print 'nbands', self._nbands
        print 'encut', self._encut 
        print 'Acell', self._Acell 
        print 'Omega', self._Omega 
        print 'Bcell', self._Bcell 
        print 'grid', self._ngrid
        print 'spin orbit', self._lsoc
#        print 'kpoints'
#        print  self._kvecs
        """
        print

    def readWFHeader(self):
        """
        Read the system information from WAVECAR, which is written in the first
        two record.

        rec1: recl, nspin, rtag
        rec2: nkpts, nbands, encut, ((cell(i,j) i=1, 3), j=1, 3)
        """

        # goto the start of the file and read the first record
        self._wfc.seek(0)
        self._recl, self._nspin, self._rtag = np.array(
            np.fromfile(self._wfc, dtype=np.float, count=3), dtype=int
        )
        self._WFPrec = self.setWFPrec()
        # the second record
        self._wfc.seek(self._recl)
        dump = np.fromfile(self._wfc, dtype=np.float, count=12)

        self._nkpts = int(dump[0])  # No. of k-points
        self._nbands = int(dump[1])  # No. of bands
        self._encut = dump[2]  # Energy cutoff
        self._Acell = dump[3:].reshape((3, 3))  # real space supercell basis
        self._Omega = np.linalg.det(self._Acell)  # real space supercell volume
        self._Bcell = np.linalg.inv(self._Acell).T  # reciprocal space supercell volume

        # Minimum FFT grid size
        Anorm = np.linalg.norm(self._Acell, axis=1)
        CUTOF = np.ceil(sqrt(self._encut / RYTOEV) / (TPI / (Anorm / AUTOA)))
        self._ngrid = np.array(2 * CUTOF + 1, dtype=int)

    def setWFPrec(self):
        """
        Set wavefunction coefficients precision:
            TAG = 45200: single precision complex, np.complex64, or complex(qs)
            TAG = 45210: double precision complex, np.complex128, or complex(q)
        """
        if self._rtag == 45200:
            return np.complex64
        elif self._rtag == 45210:
            return np.complex128
        elif self._rtag == 53300:
            raise ValueError("VASP5 WAVECAR format, not implemented yet")
        elif self._rtag == 53310:
            raise ValueError(
                "VASP5 WAVECAR format with double precision "
                + "coefficients, not implemented yet"
            )
        else:
            raise ValueError("Invalid TAG values: {}".format(self._rtag))

    def readWFBand(self, ispin=1, ikpt=1, iband=1):
        """
        Extract KS energies and Fermi occupations from WAVECAR.
        """

        self._nplws = np.zeros(self._nkpts, dtype=int)
        self._kvecs = np.zeros((self._nkpts, 3), dtype=float)
        self._bands = np.zeros((self._nspin, self._nkpts, self._nbands), dtype=float)
        self._occs = np.zeros((self._nspin, self._nkpts, self._nbands), dtype=float)

        for ii in range(self._nspin):
            for jj in range(self._nkpts):
                rec = self.whereRec(ii + 1, jj + 1, 1) - 1
                self._wfc.seek(rec * self._recl)
                dump = np.fromfile(
                    self._wfc, dtype=np.float, count=4 + 3 * self._nbands
                )
                if ii == 0:
                    self._nplws[jj] = int(dump[0])
                    self._kvecs[jj] = dump[1:4]
                dump = dump[4:].reshape((-1, 3))
                self._bands[ii, jj, :] = dump[:, 0]
                self._occs[ii, jj, :] = dump[:, 2]

        if self._nkpts > 1:
            tmp = np.linalg.norm(
                np.dot(np.diff(self._kvecs, axis=0), self._Bcell), axis=1
            )
            self._kpath = np.concatenate(([0], np.cumsum(tmp)))
        else:
            self._kpath = None
        return self._kpath, self._bands

    def gvectors(self, ikpt=1):
        """
        Generate the G-vectors that satisfies the following relation
            (G + k)**2 / 2 < ENCUT
        """
        assert 1 <= ikpt <= self._nkpts, "Invalid kpoint index!"

        kvec = self._kvecs[ikpt - 1]
        # fx, fy, fz = [fftfreq(n) * n for n in self._ngrid]
        # fftfreq in scipy.fftpack is a little different with VASP frequencies
        fx = [
            ii if ii < self._ngrid[0] / 2 + 1 else ii - self._ngrid[0]
            for ii in range(self._ngrid[0])
        ]
        fy = [
            jj if jj < self._ngrid[1] / 2 + 1 else jj - self._ngrid[1]
            for jj in range(self._ngrid[1])
        ]
        fz = [
            kk if kk < self._ngrid[2] / 2 + 1 else kk - self._ngrid[2]
            for kk in range(self._ngrid[2])
        ]
        if self._lgam:
            # parallel gamma version of VASP WAVECAR exclude some planewave
            # components, -DwNGZHalf
            kgrid = np.array(
                [
                    (fx[ii], fy[jj], fz[kk])
                    for kk in range(self._ngrid[2])
                    for jj in range(self._ngrid[1])
                    for ii in range(self._ngrid[0])
                    if (
                        (fz[kk] > 0)
                        or (fz[kk] == 0 and fy[jj] > 0)
                        or (fz[kk] == 0 and fy[jj] == 0 and fx[ii] >= 0)
                    )
                ],
                dtype=float,
            )
        else:
            kgrid = np.array(
                [
                    (fx[ii], fy[jj], fz[kk])
                    for kk in range(self._ngrid[2])
                    for jj in range(self._ngrid[1])
                    for ii in range(self._ngrid[0])
                ],
                dtype=float,
            )

        # Kinetic_Energy = (G + k)**2 / 2
        # HSQDTM    =  hbar**2/(2*ELECTRON MASS)
        KENERGY = (
            HSQDTM
            * np.linalg.norm(
                np.dot(kgrid + kvec[np.newaxis, :], TPI * self._Bcell), axis=1
            )
            ** 2
        )
        # find Gvectors where (G + k)**2 / 2 < ENCUT
        Gvec = kgrid[np.where(KENERGY < self._encut)[0]]

        if self._lsoc:
            assert Gvec.shape[0] == self._nplws[ikpt - 1] / 2, (
                "No. of planewaves not consistent for an SOC WAVECAR! %d %d %d"
                % (Gvec.shape[0], self._nplws[ikpt - 1], np.prod(self._ngrid))
            )
        else:
            assert Gvec.shape[0] == self._nplws[ikpt - 1], (
                "No. of planewaves not consistent! %d %d %d"
                % (Gvec.shape[0], self._nplws[ikpt - 1], np.prod(self._ngrid))
            )

        return np.asarray(Gvec, dtype=int)

    def save2vesta(self, phi=None, poscar="POSCAR", prefix="wfc"):
        """
        Save the real space pseudo-wavefunction as vesta format.
        """
        nx, ny, nz = phi.shape
        try:
            pos = open(poscar, "r")
            head = ""
            for line in pos:
                if line.strip():
                    head += line
                else:
                    break
            head += "\n%5d%5d%5d\n" % (nx, ny, nz)
        except:
            raise IOError("Failed to open %s" % poscar)

        with open(prefix + "_r.vasp", "w") as out:
            out.write(head)
            nwrite = 0
            for kk in range(nz):
                for jj in range(ny):
                    for ii in range(nx):
                        nwrite += 1
                        out.write("%16.8E " % phi.real[ii, jj, kk])
                        if nwrite % 10 == 0:
                            out.write("\n")
        if not self._lgam:
            with open(prefix + "_i.vasp", "w") as out:
                out.write(head)
                nwrite = 0
                for kk in range(nz):
                    for jj in range(ny):
                        for ii in range(nx):
                            nwrite += 1
                            out.write("%16.8E " % phi.imag[ii, jj, kk])
                            if nwrite % 10 == 0:
                                out.write("\n")

    def wfc_r(self, ispin=1, ikpt=1, iband=1, gvec=None, ngrid=None, norm=False):
        """
        Obtain the pseudo-wavefunction of the specified KS states in real space
        by performing FT transform on the reciprocal space planewave
        coefficients.  The 3D FT grid size is determined by ngrid, which
        defaults to self._ngrid if not given.  Gvectors of the KS states is used
        to put 1D planewave coefficients back to 3D grid.
        """
        self.checkIndex(ispin, ikpt, iband)

        if ngrid is None:
            ngrid = self._ngrid.copy()
        else:
            ngrid = np.array(ngrid, dtype=int)
            assert ngrid.shape == (3,)
            assert np.alltrue(ngrid >= self._ngrid), (
                "Minium FT grid size: (%d, %d, %d)"
                % (self._ngrid[0], self._ngrid[1], self._ngrid[2])
            )
        if gvec is None:
            gvec = self.gvectors(ikpt)

        if self._lgam:
            phi_k = np.zeros(
                (ngrid[0], ngrid[1], ngrid[2] / 2 + 1), dtype=np.complex128
            )
        else:
            phi_k = np.zeros(ngrid, dtype=np.complex128)

        gvec %= ngrid[np.newaxis, :]

        if self._lsoc:
            wfc_spinor = []
            dump = self.readBandCoeff(ispin, ikpt, iband, norm)
            nplw = dump.shape[0] / 2

            # spinor up
            phi_k[gvec[:, 0], gvec[:, 1], gvec[:, 2]] = dump[:nplw]
            wfc_spinor.append(ifftn(phi_k))
            # spinor down
            phi_k[:, :, :] = 0.0j
            phi_k[gvec[:, 0], gvec[:, 1], gvec[:, 2]] = dump[nplw:]
            wfc_spinor.append(ifftn(phi_k))

            del dump
            return wfc_spinor

        else:
            phi_k[gvec[:, 0], gvec[:, 1], gvec[:, 2]] = self.readBandCoeff(
                ispin, ikpt, iband, norm
            )

            if self._lgam:
                # add some components that are excluded and perform c2r FFT
                for ii in range(ngrid[0]):
                    for jj in range(ngrid[1]):
                        fx = ii if ii < ngrid[0] / 2 + 1 else ii - ngrid[0]
                        fy = ii if ii < ngrid[1] / 2 + 1 else ii - ngrid[1]
                        if (fy > 0) or (fy == 0 and fx >= 0):
                            continue
                        phi_k[ii, jj, 0] = phi_k[-ii, -jj, 0].conjugate()
                phi_k /= np.sqrt(2.0)
                phi_k[0, 0, 0] *= np.sqrt(2.0)
                return np.fft.irfftn(phi_k, s=ngrid)
            else:
                # perform complex2complex FFT
                return ifftn(phi_k)

    def readBandCoeff(self, ispin=1, ikpt=1, iband=1, norm=False):
        """
        Read the planewave coefficients of specified KS states.
        """

        self.checkIndex(ispin, ikpt, iband)

        rec = self.whereRec(ispin, ikpt, iband)
        self._wfc.seek(rec * self._recl)

        nplw = self._nplws[ikpt - 1]
        dump = np.fromfile(self._wfc, dtype=self._WFPrec, count=nplw)

        cg = np.asarray(dump, dtype=np.complex128)
        if norm:
            cg /= np.linalg.norm(cg)
        return cg

    def whereRec(self, ispin=1, ikpt=1, iband=1):
        """
        Return the rec position for specified KS state.
        """

        self.checkIndex(ispin, ikpt, iband)

        rec = (
            2
            + (ispin - 1) * self._nkpts * (self._nbands + 1)
            + (ikpt - 1) * (self._nbands + 1)
            + iband
        )
        return rec

    def checkIndex(self, ispin, ikpt, iband):
        """
        Check if the index is valid!
        """
        assert 1 <= ispin <= self._nspin, "Invalid spin index!"
        assert 1 <= ikpt <= self._nkpts, "Invalid kpoint index!"
        assert 1 <= iband <= self._nbands, "Invalid band index!"

    def TransitionDipoleMoment(self, ks_i, ks_j, norm=False):
        """
        calculate Transition Dipole Moment between two KS states.
        TDM in momentum representation
                                             ___              
                                  i⋅h        ╲                
        <psi_a| r | psi_b> =    --------- ⋅   ╲   Cai⋅Cbi⋅Gi
                                 Eb - Ea      ╱               
                                             ╱                
                                             ‾‾‾              
                                              i               
        Note: |psi_a> and |psi_b> should be bloch function with 
              the same k vector.

        The KS states ks_i (ks_j) is specified by list of index (ispin, ikpt, iband).
        """

        ks_i = list(ks_i)
        ks_j = list(ks_j)
        assert len(ks_i) == len(ks_j) == 3, "Must be there indexes!"
        assert ks_i[1] == ks_j[1], "k-point of the two states differ!"
        self.checkIndex(*ks_i)
        self.checkIndex(*ks_j)

        # according to the above equation, G = 0 does NOT contribute to TDM.
        gvec = np.dot(self.gvectors(ikpt=ks_i[1]), self._Bcell * TPI)
        # planewave coefficients of the two states
        phi_i = self.readBandCoeff(*ks_i, norm=norm)
        phi_j = self.readBandCoeff(*ks_j, norm=norm)
        # energy differences between the two states
        dE = (
            self._bands[ks_j[0] - 1, ks_j[1] - 1, ks_j[2] - 1]
            - self._bands[ks_i[0] - 1, ks_i[1] - 1, ks_i[2] - 1]
        )

        tmp1 = phi_i.conjugate() * phi_j
        ovlap = np.sum(tmp1)
        if self._lgam:
            tmp2 = phi_i * phi_j.conjugate()
            tdm = (
                np.sum(tmp1[:, np.newaxis] * gvec, axis=0)
                - np.sum(tmp2[:, np.newaxis] * gvec, axis=0)
            ) / 2.0
        else:
            tdm = np.sum(tmp1[:, np.newaxis] * gvec, axis=0)

        tdm = 1j / (dE / (2 * RYTOEV)) * tdm * AUTOA * AUTDEBYE

        return dE, ovlap, tdm

    def inverse_participation_ratio(self, norm=True):
        """
        Calculate Inverse Paticipation Ratio (IPR) from the wavefunction. IPR is
        a measure of the localization of Kohn-Sham states. For a particular KS
        state \phi_j, it is defined as
            
                            \sum_n |\phi_j(n)|^4 
            IPR(\phi_j) = -------------------------
                          |\sum_n |\phi_j(n)|^2||^2

        where n iters over the number of grid points.
        """

        self.ipr = np.zeros((self._nspin, self._nkpts, self._nbands, 3))

        for ispin in range(self._nspin):
            for ikpt in range(self._nkpts):
                for iband in range(self._nbands):
                    phi_j = self.wfc_r(ispin + 1, ikpt + 1, iband + 1, norm=norm)
                    phi_j_abs = np.abs(phi_j)

                    # print 'Calculating IPR of #spin %4d, #kpt %4d, #band %4d' % (ispin+1, ikpt+1, iband+1)
                    self.ipr[ispin, ikpt, iband, 0] = (
                        self._kpath[ikpt] if self._kpath is None else 0
                    )
                    self.ipr[ispin, ikpt, iband, 1] = self._bands[ispin, ikpt, iband]
                    self.ipr[ispin, ikpt, iband, 2] = (
                        np.sum(phi_j_abs ** 4) / np.sum(phi_j_abs ** 2) ** 2
                    )

        np.save("ipr.npy", self.ipr)
        return self.ipr


############################################################

if __name__ == "__main__":
    # xx = vaspwfc('wavecar')
    # phi = xx.wfc_r(1, 30, 17, ngrid=(28, 28, 252))
    # xx.save2vesta(phi, poscar='POSCAR')

    # xx = vaspwfc('./gamma/WAVECAR')
    # phi = xx.wfc_r(1, 1, 317, ngrid=(60, 108, 160),
    #                gamma=True)
    # xx.save2vesta(phi, poscar='./gamma/POSCAR',gamma=True)

    # xx = vaspwfc('WAVECAR')
    # dE, ovlap, tdm = xx.TransitionDipoleMoment([1,30,17], [1,30,18], norm=True)
    # print dE, ovlap.real, np.abs(tdm)**2

    # print xx._recl, xx._nspin, xx._rtag
    # print xx._nkpts, xx._nbands, xx._encut
    # print xx._Acell, xx._Bcell
    # # print np.linalg.norm(xx._Acell, axis=1)
    # print xx._ngrid
    # print xx._bands[0,0,:]
    # print xx._kvecs
    # print xx._kpath
    # b = xx.readBandCoeff(1,1,1)
    # xx = np.savetxt('kaka.dat', xx.gvectors(2), fmt='%5d')
    # gvec = xx.gvectors(1)
    # gvec %= xx._ngrid[np.newaxis, :]
    # print gvec

    # ngrid=(28, 28, 252)
    # phi = xx.wfc_r(1, 30, 17, ngrid=(28, 28, 252))
    # header = open('POSCAR').read()
    # with open('wave_real.vasp', 'w') as out:
    #     out.write(header)
    #     out.write('%5d%5d%5d\n' % (ngrid[0], ngrid[1], ngrid[2]))
    #     nwrite=0
    #     for kk in range(ngrid[2]):
    #         for jj in range(ngrid[1]):
    #             for ii in range(ngrid[0]):
    #                 nwrite += 1
    #                 out.write('%22.16f ' % phi.real[ii,jj,kk])
    #                 if nwrite % 10 == 0:
    #                     out.write('\n')
    # with open('wave_imag.vasp', 'w') as out:
    #     out.write(header)
    #     out.write('%5d%5d%5d\n' % (ngrid[0], ngrid[1], ngrid[2]))
    #     nwrite=0
    #     for kk in range(ngrid[2]):
    #         for jj in range(ngrid[1]):
    #             for ii in range(ngrid[0]):
    #                 nwrite += 1
    #                 out.write('%22.16f ' % phi.imag[ii,jj,kk])
    #                 if nwrite % 10 == 0:
    #                     out.write('\n')

    # xx = vaspwfc('wave_tyz')
    # ipr = xx.inverse_participation_ratio()
    # print xx._nbands, xx._nkpts
    #
    # import matplotlib as mpl
    # import matplotlib.pyplot as plt
    #
    # fig = plt.figure()
    # ax = plt.subplot()
    #
    # ax.scatter(ipr[...,0], ipr[..., 1], s=ipr[..., 2] / ipr[..., 2].max() * 10, c=ipr[..., 2],
    #            cmap='jet_r')
    #
    # plt.show()

    xx = vaspwfc("examples/wfc_r/wavecar_mose2-wse2", lsorbit=True)
    phi_spinor = xx.wfc_r(1, 1, 36, ngrid=xx._ngrid * 2)
    for ii in range(2):
        phi = phi_spinor[ii]
        prefix = "spinor_{:02d}".format(ii)
        xx.save2vesta(phi, prefix=prefix, poscar="examples/wfc_r/poscar_mose2-wse2")

    pass



import glob, json

mem = []
for file in glob.glob("*.alloy"):
    ff = str(file)
    element_ff = []
    f = open(ff, "r")
    list_el = []
    lines = f.readlines()
    content = (lines[3]).split(" ")
    # content=(lines[3]).split("' '|\n|\r\n")
    for val in content:

        if val != "" and val != "\n" and val != "\r\n":
            list_el.append(val)
    for i in range(0, len(list_el)):
        if i != 0:
            # element_ff.append(list_el[i])
            element_ff.append(list_el[i].split("\n")[0])
    print(file, element_ff)
    info = {}
    info["elements"] = element_ff
    info["file"] = file
    mem.append(info)
print(len(mem))
f = open("alloy_pots.json", "w")
f.write(json.dumps(mem, indent=4))
f.close()



"""
Integration of JARVIS-FF/DFT/ML with genetic algorithm.
"""
import os

# env=str(os.path.join(os.path.dirname(__file__),'env_variables'))
try:
    dir = os.path.join(os.path.dirname(__file__))
    env = os.path.join(dir, "env_variables")
    # env=str('env_variables'))
    f = open(env, "r")
    lines = f.read().splitlines()
    f.close()
    for i in lines:
        tmp = i.split("=")
        if len(tmp) > 1:
            # print (tmp[0],tmp[1])
            os.environ[tmp[0]] = tmp[1]
except:
    print("Cannot find env_variables in site-packages/path")
    pass



import numpy as np
from pymatgen.core.structure import Structure

"""
Coulomb matrix for a structure
Refer: 10.1103/PhysRevLett.108.058301
"""


def coulomb_matrix(strt="", max_dim=100):
    """
    Args:
        strt: structure object
        max_dim: maximum number of atoms=sqrt(max_dim)
    Returns:
          z: numpy array of 1 x max_dim dimension
    """
    natoms = len(strt)
    mat = np.zeros((natoms, natoms))
    for ii, i in enumerate(strt):
        for jj, j in enumerate(strt):
            if ii == jj:
                mat[ii, jj] = 0.5 * i.specie.Z ** 2.4
            else:
                dist = strt.get_distance(ii, jj)
                mat[ii, jj] = (i.specie.Z * j.specie.Z) / dist
    tmp = mat.ravel()
    z = np.pad(tmp, (0, max_dim), "constant")
    return z


if __name__ == "__main__":
    s = Structure.from_file("POSCAR")
    coulomb_matrix(strt=s)
